/********************************************************************
*  File:    UTIL.C                                                  *
*  Purpose: Comtrol C Programmer's Toolkit Hostess i Coprocessor    *
*           Board Control Utilities                                 *
*  Date:    November 7, 1990                                        *
*  Author:  Howard D. Straus                                        *
*  Release: 1.00 - Original release                                   *
***********************************************************************
*                         COPYRIGHT NOTICE                            *
*=====================================================================*
*   Copyright 1990, 1991 by Howard D. Straus and Comtrol Corporation. *
*   All rights reserved.  Use or reproduction by any means of any     *
*   portion of this code contrary to the developers license agreement *
*   is expressly prohibited.                                          *
********************************************************************/

#include "compiler.h"

#include <ctype.h>
#include <dos.h>
#ifdef  TURBO_C
#include <graphics.h>
#else
#include <graph.h>
#endif
#include <string.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <fcntl.h>
#include <io.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "..\hostess\hostess.h"

#include "host.h"
#include "hostfns.h"
#include "hostext.h"

/********************************************************************
*  Name:            atoh
*  Purpose:         convert a hexadecimal ASCII string to a long
*       integer.
*  Call:            l = atoh(s);
*  Parameters:
*       char *s;  the string of hex characters to convert
*  Returns:
*       ULONG l;  the integer result of the conversion
*  Method:          converts the characters one at a time to hexa-
*       decimal.  the first non-hexadecimal character terminates the
*       conversion.
*  Notes:
********************************************************************/

ULONG
atoh(a)
        char            *a;     /* hex string to convert to integer */
{   ULONG           r;      /* return value */

        r = 0;
        while (isxdigit(*a))
        {   r = (r << 4) | (*a <= '9' ? *a & 0xF : toupper(*a) - 'A' + 10);
                a++;
        }
        return(r);
}


/********************************************************************
*  Name:        begin_hcb_execution
*  Purpose:     start the Hostess i coprocessor board executing
*       the uploaded program
*  Call:        r = begin_hcb_execution();
*  Parameters:  none.
*  Returns:
*       UINT r;  the result of the operation: 0 = success, -1 = fail
*  Method:      the interaction flag is written to a known value, then
*       the board is given an interrupt telling it to begin execution
*       of the program.  When the interaction flag returns to the ACK
*       value, the operation is complete.  If the flag does not return
*       to the ACK value within 4 seconds, a failure code is returned.
*  Notes:       the base port and the shared memory address must be
*       properly set before this operation is attempted.
********************************************************************/

UINT
begin_hcb_execution()
{   unsigned int test_word;             /* local test word */
        time_t  tick;                       /* time hold word */

        test_word = ~HCB_ACK_VALUE;       /* set the interaction flag */
        write_to_hcb(base_address + HCB_RESET_WORD_OFFSET,
                                        (char far *)&test_word, 1);
        OUTP(base_port + HCB_START_PORT_OFFSET, 0); /* interrupt Hostess i */
        time(&tick);                        /* get start time */
        while (test_word != TOOLKIT_RUNNING_VALUE)
                                                                                /* wait for Hostess i to complete */
        {   read_from_hcb((char far *)&test_word, /* pick up interaction word */
                                                 base_address + HCB_RESET_WORD_OFFSET, 1);
                if ((time(NULL) - tick) > 6)    /* if no response in 6 seconds */
                {   show_output("<< Hostess Board failed to respond. >>\r\n");
                        return(-1);                 /* error return */
                }
        }
        return(0);
}


/********************************************************************
*  Name:        configure_port
*  Purpose:     set up a Hostess i communications port
*  Call:        r = configure_port();
*  Parameters:  none.
*  Returns:
*       int r;  completion status: 0 = success, -1 = failure
*  Method:      the port parameters from global memory are placed into
*       an appropriate message structure which is then queued to the
*       host output queue for transmission.
*  Notes:       the global variables which are sent are:
*       port_speed, data_bits, parity, stop_bits, and loopback_mode.
*       They must be set before this function is called.
********************************************************************/

int
configure_port()
{   BH_     *bh;                        /* buffer pointer */
        MG_     *mg;                        /* message structure pointer */
        CD_     *cd;                        /* device configuration structure */

        if ((bh = hq_alc()) == NULL)        /* get a buffer */
        {   show_output("<< Buffer shortage, cannot configure port. >>\r\n");
                return(-1);                     /* none available */
        }
        mg = (MG_ *)bh->bh_text;            /* place message in buffer */
        mg->mg_type = CONFIG_PORT;          /* set message type */
        mg->mg_line = hcb_port;             /* designate which port */
        cd = (CD_ *)mg->mg_data;            /* imbed device config data */
        cd->cd_spd = (port_speed == 9600 ? 0xFFFF : port_speed * 10);
                                                                                /* send port speed */
        cd->cd_bits = data_bits;            /* send data bits per character */
        cd->cd_par = parity;                /* send character parity */
        cd->cd_stpb = (stop_bits - 1) << 2; /* send character stop bits */
        cd->cd_loop = (loopback_mode ? 0x10 : 0); /* send loopback mode bit */
        bh->bh_txct = sizeof(CD_) + MG_HDR; /* message size */
        hq_enq(bh, &output_qcb);            /* add to the host output queue */
        return(0);                          /* return success */
}


/********************************************************************
*  Name:            enter_debugger
*  Purpose:         sends a message to the Hostess i which causes it
*       to enter its on-board debugger.
*  Call:            enter_debugger();
*  Parameters:      none.
*  Returns:         none.
*  Method:          composes a message buffer to the Hostess i, then
*       queues it to the output queue.
*  Notes:           if there is no buffer available, the command is
*       ignored, and a diagnostic message is sent to the output screen.
********************************************************************/

void
enter_debugger()
{   BH_             *bh;
        MG_             *mg;

        if ((bh = hq_alc()) == NULL)        /* get a message buffer */
        {   show_output("<< No message buffers available >>\r\n");
                return;
        }
        mg = (MG_ *)bh->bh_text;            /* point to the data portion */

        mg->mg_type = ENTER_DEBUGGER;       /* set message type */
        bh->bh_txct = ENTER_DEBUGGER_SIZE;  /* set the message size */
        hq_enq(bh, &output_qcb);            /* queue for output */
#ifdef  VERBOSE
        show_output("<< Enter debugger sent. >>\r\n");
#endif
}

/********************************************************************
*  Name:            host_buffer_count
*  Purpose:         returns the count of characters in the Hostess i
*       host communications buffer.
*  Call:            ct = host_buffer_count();
*  Parameters:      none.
*  Returns:
*       int ct;  the number of characters in the Hostess i host
*               communications buffer.
*  Method:          reads the Hostess i mailbox, and returns the host
*       semaphore value.
*  Notes:
********************************************************************/

UINT
host_buffer_count()
{   MB_     mailbox;

        read_from_hcb((char far *)&mailbox, mailbox_addr, 4);
        return(mailbox.mb_hoct);
}


/********************************************************************
*  Name:        open_line
*  Purpose:     open a Hostess i communications port for operation
*  Call:        r = open_line(io);
*  Parameters:
*               int io;  input/output indicator: bit 0 - set for input ops
*                                                                                        bit 1 - set for output ops
*  Returns:
*       int r;  completion status: 0 = success, -1 = failure
*  Method:              the current port from global memory is activated for
*               either input or output operations, or both.  data may then
*               be sent to and/or received from the port.
*  Notes:
********************************************************************/

int
open_line(io)
        int             io;                                     /* 1 = input, 2 = output, 3 = both */
{   BH_     *bh;                        /* buffer pointer */
        MG_     *mg;                        /* message structure pointer */
        CD_     *cd;                        /* device configuration structure */

        if ((bh = hq_alc()) == NULL)        /* get a buffer */
        {   show_output("<< Buffer shortage, cannot open line. >>\r\n");
                return(-1);                     /* none available */
        }
        mg = (MG_ *)bh->bh_text;            /* place message in buffer */
        mg->mg_type = OPEN_LINE;            /* set message type */
        mg->mg_line = hcb_port;             /* designate which port */
        mg->mg_data[0] = io;                    /* send the I/O indicator */
        bh->bh_txct = 1 + MG_HDR;                       /* message size */
        hq_enq(bh, &output_qcb);            /* add to the host output queue */
        return(0);                          /* return success */
}


/********************************************************************
*  Name:            hcb_buffer_count
*  Purpose:         returns the count of characters in the Hostess i
*       message communications buffer.
*  Call:            ct = hcb_buffer_count();
*  Parameters:      none.
*  Returns:
*       int ct;  the number of characters in the Hostess i message
*               communications buffer.
*  Method:          reads the Hostess i mailbox, and returns the
*       message buffer semaphore/count value.
*  Notes:
********************************************************************/

UINT
hcb_buffer_count()
{   MB_     mailbox;

        read_from_hcb((char far *)&mailbox, mailbox_addr, 4);
        return(mailbox.mb_ulct);
}


/********************************************************************
*  Name:            load_hcb
*  Purpose:         loads the Hostess i board with the program
*       contained in the file whose name is in hcb_load_file_name.
*  Call:            err = load_hcb(exe_file_path);
*  Parameters:
*       char *exe_file_path;  file path of the Hostess i executable
*                   file to be loaded.
*  Returns:
*       UINT err;  error indicator: 0 = successful, -1 = error.
*  Method:      allocates enough memory to read the entire load
*       file, reads the file into that memory, and transfers the
*       relevant portion of the ".EXE" file into the appropriate
*       location in the Hostess i dual-ported memory.
*  Notes:       512 bytes at the beginning of an EXE file are not
*       executable code, and must be discarded.  Since the code is
*       origin'ed at 0x80 (128) bytes from zero, an additional 128
*       bytes of unused file must also be discarded, for a total of
*       512 + 128 = 640 bytes.  This number is a constant value de-
*       fined in HOST.H as THROWAWAY_BYTES.
********************************************************************/

UINT
load_hcb(file_path)
        char            *file_path;
{   int             exe_file;
        int             bytes_read;
        char    far     *cp;
        char            *mp;
        ULONG           load_point;
        int             throwaway;
        UINT            total_load;
        char            cvtstr[10];
        char            total_string[40];

        if ((exe_file = open(file_path, O_BINARY | O_RDONLY)) == -1)
        {   show_output("<< Unable to open executable file. >>\r\n");
                return(-1);
        }
        total_load = 0L;                    /* initialize total loaded bytes */
        throwaway = THROWAWAY_BYTES;        /* strip beginning of .EXE file */
        mp = (char *)malloc(1024);          /* allocate memory for read */
        load_point = base_address + HIGH_PAGE + 0x80; /* initial load point */
        while (!eof(exe_file))
        {   if ((bytes_read = read(exe_file, mp, 1024)) <= 0) /* read the file */
                {   show_output("<< Read error on executable file. >>\r\n");
                        return(-1);                 /* return error status */
                }
                if (throwaway < bytes_read)
                {   cp = (char far *)mp + throwaway; /* discard file beginning */
                        bytes_read -= throwaway;
                        throwaway = 0;                          /* no more to discard */
                        write_to_hcb(load_point, cp, bytes_read);
                                                                                /* upload the file chunk */
                        load_point += bytes_read;       /* move load point up */
                        total_load += bytes_read;   /* tally total load */
                }
                throwaway = max(throwaway - bytes_read, 0);
        }
        close(exe_file);                    /* close the file */
        free(mp);                           /* free the memory used */
        strcpy(total_string, "<< ");
        strcat(total_string, itoa(total_load, cvtstr, 10));
        strcat(total_string, " bytes loaded to Hostess. >>\r\n");
        show_output(total_string);
        return(0);
}


/********************************************************************
*  Name:            mem_xfer
*  Purpose:         transfer memory between PC and Hostess i dual-ported
*       memory.
*  Call:            r = mem_xfer(source, destination, length);
*  Parameters:
*       ULONG source;  the absolute address of the source buffer.
*       ULONG destination;  the absolute address of the destination
*                   buffer.
*       UINT length;  the number of words to transfer.
*  Returns:
*       UINT r;  the return code of the transfer.
*                0 = successful transfer.
*                1 = RAM parity error.
*                2 = exception interrupt error.
*                3 = gate address line 20h failed.
*  Method:          sets up the Global Descriptor Table entries, then
*       uses INT 15, function 87 (hex) to transfer the data between
*       the memory buffers.
*  Notes:
********************************************************************/

UINT
mem_xfer(destination, source, length)
        ULONG           destination;
        ULONG           source;
        UINT            length;
{   GDT_            *sd;            /* source GDT address */
        GDT_            *dd;            /* target GDT address */
        UINT            gd_table[6][4]; /* GDT array for transfer */
        union   REGS    inregs, outregs;
        struct  SREGS   segregs;
        UINT    far     *fi;

        memset(gd_table, 0, sizeof(gd_table));  /* clear Global Descriptor Tab */

        sd = (GDT_ *)gd_table[2];               /* point to source GDT */
        sd->gd_addr = source | 0x93000000L;     /* set address and access byte */
        sd->gd_sglh = 0xFFFF;                   /* maximum length */

        dd = (GDT_ *)gd_table[3];               /* point to destination GDT */
        dd->gd_addr = destination | 0x93000000L; /* set address and access byte */
        dd->gd_sglh = 0xFFFF;                   /* maximum length */

        inregs.h.ah = 0x87;                     /* function 0x87 */
        inregs.x.cx = length;                   /* transfer length */

        fi = (UINT far *)gd_table[0];           /* pointer to GDT */
        inregs.x.si = FP_OFF(fi);               /* to registers */
        segregs.es = FP_SEG(fi);
        int86x(0x15, &inregs, &outregs, &segregs); /* execute function */
        return((unsigned)outregs.h.ah);         /* return result */
}


/********************************************************************
*  Name:            norm
*  Purpose:         compute the absolute address pointed to by a far
*           pointer.
*  Call:            abs_addr = norm(fp);
*  Parameters:
*       void far *fp;  far pointer to convert to absolute memory addr.
*  Returns:
*       unsigned long abs_addr;  the absolute address pointed to by
*               the segment:offset combination fp.
*  Method:      decomposes the segment and offset, and adds them to-
*       gether, the segment shifted left 4 bits.
*  Notes:
********************************************************************/

ULONG
norm(fp)
        void    far     *fp;
{
        return(((ULONG)FP_SEG(fp) << 4) + (ULONG)FP_OFF(fp));
}


/********************************************************************
*  Name:            read_from_hcb
*  Purpose:         pass a buffer to the Hostess i board memory
*  Call:            read_from_hcb(buffer, address, length);
*  Parameters:
*       char far *buffer;  pointer to the buffer into which to read
*           the characters from the Hostess i.
*       ULONG address;  the absolute memory address from which to read
*       UINT length;  the number of characters to read
*  Returns:
*  Method:          uses mem_xfer to transfer data from the Hostess i dual
*       ported memory to a local buffer.
*  Notes:
********************************************************************/

UINT
read_from_hcb(buffer, address, length)
        char    far     *buffer;
        ULONG           address;
        UINT            length;
{
        return(mem_xfer(norm(buffer), address, length));
}


/********************************************************************
*  Name:            reset_hcb
*  Purpose:         causes a general reset of the Hostess i board, then
*       waits until the board has completed its initialization sequence.
*  Call:            r = reset_hcb();
*  Parameters:      none.
*  Returns:         none.
*       int r;  result of operation: 0 if success, -1 if failure.
*  Method:          writes the required data to the port offset from
*       the base port required to send a reset to the Hostess i board.
*       Waits until the flag word indicates a successful initialization
*       sequence.  If the board has not indicated its presence after
*       seven seconds, a failure code is returned.
*  Notes:           the base address of the board and the base port
*       number must have already been set prior to calling reset_hcb.
********************************************************************/

int
reset_hcb()
{   UINT            reset_flag;
        time_t          reset_start;
        int             i;

        reset_flag = 0;                     /* initialize flag */
        OUTP(base_port + HCB_RESET_PORT_OFFSET, 0x00); /* set signal */
        DELAY(100);                         /* wait 100 msec. */
        OUTP(base_port + HCB_RESET_PORT_OFFSET, 0xFF); /* clear signal */

        /*
                set up the Hostess i memory map above 1 Meg.
                transfer mode = AT (16-bit transfers)
        */
        OUTP(base_port + HCB_CTL_REGS_SELECT_PORT_OFFSET,
                                         CTL_REG_1_SELECT + DISABLE_DP_RAM);
        i = (base_address >> 17) + AT_MEM_XFERS;
        OUTP(base_port + HCB_CTL_REGS_PORT_OFFSET, i);

        /*
                set both control registers 2 and 3 to 0 because we are using
                dual ported (DP) RAM above 1 Mbyte.
        */
        OUTP(base_port + HCB_CTL_REGS_SELECT_PORT_OFFSET,
                                         CTL_REG_2_SELECT + DISABLE_DP_RAM);
        OUTP(base_port + HCB_CTL_REGS_PORT_OFFSET, 0);

        OUTP(base_port + HCB_CTL_REGS_SELECT_PORT_OFFSET,
                                         CTL_REG_3_SELECT + DISABLE_DP_RAM);
        OUTP(base_port + HCB_CTL_REGS_PORT_OFFSET, 0);

        /*
                set up the IRQ number to use for interrupts from the Hostess i
        */
        OUTP(base_port + HCB_CTL_REGS_SELECT_PORT_OFFSET,
                                         CTL_REG_4_SELECT + DISABLE_DP_RAM);
        OUTP(base_port + HCB_CTL_REGS_PORT_OFFSET, irq_select[irq_no]);

        /*
                enable Dual-Ported RAM
        */
        OUTP(base_port + HCB_MEM_ENABLE_PORT_OFFSET, ENABLE_DP_RAM);
                                                                                /* enable Hostess i DP-RAM */
        OUTP(base_port + HCB_CTL_REGS_SELECT_PORT_OFFSET,
                                         CTL_REG_3_SELECT + ENABLE_DP_RAM);
        time(&reset_start);                 /* get current time */
        while (reset_flag != HCB_ACK_VALUE) /* wait for Hostess to ACK */
        {   i = read_from_hcb((char far *)&reset_flag,
                                        base_address + HCB_RESET_WORD_OFFSET, 1);
                if ((time(NULL) - reset_start) > 7) /* if no ACK for 7 seconds */
                {   show_output("<< Hostess failed to reset.>>\r\n");
                        return(-1);                 /* board failed to respond */
                }
        }
        show_output("<< Hostess Board reset. >>\r\n"); /* announce success */
        return(0);                          /* return success */
}


/********************************************************************
*  Name:            send_char_to_hcb
*  Purpose:         send a character to the Hostess coprocessor board
*  Call:            send_char_to_hcb(c);
*  Parameters:
*       char c;  the character to be sent to the Hostess for transmis-
*           sion via the selected output port.
*  Returns:         none.
*  Method:          saves the new character to the keyboard buffer.
*       if the count is above a threshhold, or one second has passed
*       since the last buffer was sent, transfers the buffer to the
*       Hostess.  The character is sent to the output window on the
*       display.
*  Notes:
********************************************************************/

void
send_char_to_hcb(c)
        char            c;                  /* the character to send */
{   UTINY           send_flag;          /* send the buffer indicator flag */
        time_t          now;                /* current DOS time */
        char                    *echo_ptr;                      /* echo pointer */

        send_flag = FALSE;                  /* initialize the send flag */
        echo_ptr = kp;                                          /* place to echo from */
        *kp++ = c;                          /* buffer the character */
        if (strchr("\003\004\r\n", c) != NULL) /* if terminal character */
                send_flag = TRUE;               /* send on CR, LF, ETX, or EOT */
        if (strlen(keyboard_buffer) > 50)   /* if 50 or more characters in buf */
                send_flag = TRUE;               /* send the buffer */
        if (time(&now) != last_second)      /* if a second has elapsed */
        {   last_second = now;              /* update time holder */
                send_flag = TRUE;               /* send the buffer */
        }
        show_output(echo_ptr);                          /* echo stored character[s] */
        if (c == '\r')                                          /* if character was CR */
                show_output("\n");                              /* output a LF */
        if (send_flag)                      /* buffer to be sent */
        {   if (!send_string_to_hcb(keyboard_buffer, NO_ECHO)) /* send it */
                {   memset(keyboard_buffer, 0, sizeof(keyboard_buffer));
                                                                                /* clear the keyboard buffer */
                        kp = keyboard_buffer;       /* reset pointer */
                }
        }
}


/********************************************************************
*  Name:            send_file
*  Purpose:         send an ASCII file to the Hostess comm port
*  Call:            r = send_file();
*  Parameters:      none.
*  Returns:
*       int r;  completion code: 0 = success, -1 = fail.
*  Method:          reads in the designated message file, buffer by
*       buffer, uses send_string_to_hcb to queue each buffer to
*       the host output queue.
*  Notes:           there must be enough buffers available to accom-
*       modate the entire text file, or the message will be truncated.
********************************************************************/

int
send_file()
{   FILE    *tx_file;               /* file handle */
        int     bytes_read;             /* file read result */
        char    buffer[BUF_SIZE];       /* buffer for reading file */

        if ((tx_file = fopen(message_file_name, "r")) == NULL)
        {   show_output("<< Unable to open Tx message file. >>\r\n");
                fclose(tx_file);                /* close the file */
                return(-1);                                     /* return error */
        }
        while (!feof(tx_file))          /* send the whole file */
        {   memset(buffer, 0, sizeof(buffer)); /* clear the buffer */
                if (fgets(buffer, 79, tx_file) == NULL) /* get the next line */
                        break;                                  /* error or EOF */
                if (send_string_to_hcb(buffer, ECHO)) /* send data to Hostess port */
                        return(-1);             /* error occurred, return status */
                if (buffer[strlen(buffer) - 1] == '\n')
                        show_output("\r");              /* for display vagary */
        }
        fclose(tx_file);                /* close the file */
        return(0);                      /* return success */
}


/********************************************************************
*  Name:            send_string_to_hcb
*  Purpose:         send a character string to the Hostess for trans-
*       mission via the selected port
*  Call:            err = send_string_to_hcb(s, echo);
*  Parameters:
*       char *s;  the string to be sent to the Hostess.
*       int echo;  != 0: echo the string to the output window.
*                  == 0: no echo to the output window
*  Returns:
*       int err;  error indication: 0 = successful, -1 = unsuccessful
*  Method:          a free buffer is fetched from the free queue, and
*       the string copied into its data area.  the buffer is then
*       added to the output queue, the queue which is waiting to be
*       sent to the Hostess.  if an echo is requested, the string is
*       also displayed in the output window.
*  Notes:
********************************************************************/

int
send_string_to_hcb(s, echo)
        char            *s;                 /* string to send */
        int             echo;               /* if true, echo string to window */
{   MG_             *mg;                /* message pointer */
        BH_             *bh;                /* queue buffer pointer */

        if ((bh = hq_alc()) == NULL)        /* no buffers available */
        {   show_output("<< Out of buffers in send string. >>\r\n");
                return(-1);                     /* return error */
        }
        mg = (MG_ *)bh->bh_text;            /* where to put message */
        mg->mg_type = WRITE_DATA;           /* indicate data to write out */
        mg->mg_line = hcb_port;           /* comm port to which to send */
        bh->bh_txct = strlen(s) + MG_HDR;   /* set message length */
        strcpy(mg->mg_data, s);             /* copy string to buffer */
        hq_enq(bh, &output_qcb);            /* enqueue buffer for output */
        if (echo)                           /* if echo requested */
                show_output(s);                 /* show in output window */
        return(0);                          /* return no error */
}


/********************************************************************
*  Name:            set_time
*  Purpose:         set the time on the Hostess board to the same as the
*           time on the PC
*  Call:            set_time();
*  Parameters:      none.
*  Returns:         status of Hostess write (0 = success).
*  Method:          sets up the SET__TIME message, then sends it to
*           the Hostess Host communications buffer.
*  Notes:           The host count must be zero when this routine is
*           called, as it does no checking.  The host buffer address
*           must have been initialized.
********************************************************************/

void
set_time()
{   BH_             *bh;
        MG_             *mg;

        if ((bh = hq_alc()) == NULL)        /* get a free buffer */
        {   set_time_flag = TRUE;           /* set deferred time flag */
#ifdef  VERBOSE
                show_output("<< Set time deferred. >>\r\n");
#endif
                return;
        }
        mg = (MG_ *)bh->bh_text;            /* point to the data portion */

        mg->mg_type = SET_TIME;             /* set message type */
        time((time_t *)mg->mg_data);        /* insert the time */
        bh->bh_txct = SET_TIME_SIZE;        /* set the message size */
        hq_enq(bh, &output_qcb);            /* queue for output */
#ifdef  VERBOSE
        show_output("<< Set time sent. >>\r\n");
#endif
        set_time_flag = FALSE;              /* clear deferred time flag */
}


/********************************************************************
*  Name:            send_modem_control
*  Purpose:         send a modem control command to the Hostess
*  Call:            send_modem_control(mcw);
*  Parameters:
*       UINT mcw;  a word containing the RTS and DTR request bits.
*               RTS is bit 9, DTR is bit 15.  If the bit is set, the
*               signal will be turned "on" by the Hostess, else, it
*               will be turned "off".
*  Returns:         none.
*  Method:          a buffer is fetched for the message.  if there is
*       no buffer available, the modem control request word is saved
*       and a flag is set to indicate a deferred message is waiting.
*       otherwise, the modem control message is constructed and put
*       on the output queue.
*  Notes:           the host monitor must check the deferred message
*       flag in its monitor loop.
********************************************************************/

void
send_modem_control(mcw)
        UINT        mcw;                    /* RTS and DTR signals */
{   BH_         *bh;                    /* buffer pointer */
        MG_         *mg;                    /* message text pointer */

        if ((bh = hq_alc()) == NULL)        /* no buffers available */
        {   modem_control_flag = TRUE;      /* deferred modem control change */
                modem_control_word = mcw;       /* save modem control word */
#ifdef  VERBOSE
                show_output("<< Modem control deferred. >>\r\n");
#endif
                return;
        }
        mg = (MG_ *)bh->bh_text;            /* point to text */
        mg->mg_type = MODEM_CONTROL;        /* message type */
        mg->mg_data[0] = (mcw >> 8);        /* send control byte */
        mg->mg_line = hcb_port;           /* comm port for which to set */
        bh->bh_txct = 1 + MG_HDR;           /* length of message */
        hq_enq(bh, &output_qcb);            /* add buffer to queue */
        modem_control_flag = FALSE;         /* turn off deferred modem control */
#ifdef  VERBOSE
        show_output("<< Modem control sent. >>\r\n");
#endif
}


/********************************************************************
*  Name:            write_to_hcb
*  Purpose:         pass a buffer to the Hostess board memory
*  Call:            write_to_hcb(address, buffer, length);
*  Parameters:
*       ULONG address;  the absolute memory address to which to write
*       char far *buffer;  pointer to the buffer to write to the Hostess
*       UINT length;  the number of words to write
*  Returns:
*       UINT r;  the result of the transfer.
*  Method:          uses mem_xfer to transfer data from local memory
*       to the Hostess dual-ported memory space.
*  Notes:
********************************************************************/

unsigned
write_to_hcb(destination, buffer, length)
        ULONG           destination;    /* absolute address of Hostess buffer */
        char    far     *buffer;
        UINT            length;
{
        return(mem_xfer(destination, norm((char far *)buffer), length));
}


#ifndef TURBO_C
/********************************************************************
*  Name:                wait
*  Purpose:             delays for requested milliseconds; emulates Turbo C
*           function
*  Call:                wait(msec);
*  Parameters:
*               int msec;  number of milliseconds to wait.
*  Returns:             none.
*  Method:              takes the time, waits until msec milliseconds have
*               passed, then returns.
*  Notes:
********************************************************************/

#include <sys\types.h>
#include <sys\timeb.h>

void
wait(msec)
        int             msec;
{   struct      timeb tick, tock;

        ftime(&tock);                           /* fetch time */
        do
        {       ftime(&tick);   /* check time */
        } while ((difftime(tick.time, tock.time) * 1000 +
                                tick.millitm - tock.millitm) < msec);
}
#endif


                                        /* ***** END OF FILE ***** */
